import contextlib

import nonebot
from nonebot.adapters import Bot
from nonebot_plugin_uninfo import get_interface

from ..log import logger
from ..models.group_console import GroupConsole
from ..models.plugin_info import PluginInfo
from .zxpm_ban import *  # noqa: F403
from .zxpm_help import *  # noqa: F403
from .zxpm_hooks import *  # noqa: F403
from .zxpm_init import *  # noqa: F403
from .zxpm_plugin_switch import *  # noqa: F403
from .zxpm_set_admin import *  # noqa: F403

driver = nonebot.get_driver()

with contextlib.suppress(ImportError):
    from nonebot.adapters.onebot.v11 import GroupIncreaseNoticeEvent  # noqa: F401

    from .zxpm_add_group import *  # noqa: F403
    from .zxpm_admin_watch import *  # noqa: F403


@driver.on_bot_connect
async def _(bot: Bot):
    """更新bot群组信息

    参数:
        bot: Bot
    """
    try:
        if interface := get_interface(bot):
            scens = await interface.get_scenes()
            group_list = [(s.id, s.name) for s in scens if s.is_group]
            db_group_list = await GroupConsole.all().values_list("group_id", flat=True)
            block_modules = await PluginInfo.filter(
                load_status=True, default_status=False
            ).values_list("module", flat=True)
            block_modules = [f"<{module}" for module in block_modules]
            create_list = []
            for gid, name in group_list:
                if gid not in db_group_list:
                    group = GroupConsole(group_id=gid, group_name=name)
                    if block_modules:
                        group.block_plugin = ",".join(block_modules) + ","
                    logger.debug(f"Bot: {bot.self_id} 添加创建群组Id: {group.group_id}")
                    create_list.append(group)
            if create_list:
                await GroupConsole.bulk_create(create_list, 10)
                logger.debug(
                    f"更新Bot: {bot.self_id} 共创建 {len(create_list)} 条群组数据..."
                )
    except Exception as e:
        logger.error(f"获取Bot: {bot.self_id} 群组发生错误...", e=e)



from tortoise import fields

from ..db_connect import Model


class LevelUser(Model):
    id = fields.IntField(pk=True, generated=True, auto_increment=True)
    """自增id"""
    user_id = fields.CharField(255)
    """用户id"""
    group_id = fields.CharField(255)
    """群聊id"""
    user_level = fields.BigIntField()
    """用户权限等级"""
    group_flag = fields.IntField(default=0)
    """特殊标记，是否随群管理员变更而设置权限"""

    class Meta:  # type: ignore
        table = "zxpm_level_users"
        table_description = "用户权限数据库"
        unique_together = ("user_id", "group_id")

    @classmethod
    async def get_user_level(cls, user_id: str, group_id: str | None) -> int:
        """获取用户在群内的等级

        参数:
            user_id: 用户id
            group_id: 群组id

        返回:
            int: 权限等级
        """
        if not group_id:
            return 0
        if user := await cls.get_or_none(user_id=user_id, group_id=group_id):
            return user.user_level
        return 0

    @classmethod
    async def set_level(
        cls,
        user_id: str,
        group_id: str,
        level: int,
        group_flag: int = 0,
    ):
        """设置用户在群内的权限

        参数:
            user_id: 用户id
            group_id: 群组id
            level: 权限等级
            group_flag: 是否被自动更新刷新权限 0:是, 1:否.
        """
        await cls.update_or_create(
            user_id=user_id,
            group_id=group_id,
            defaults={
                "user_level": level,
                "group_flag": group_flag,
            },
        )

    @classmethod
    async def delete_level(cls, user_id: str, group_id: str) -> bool:
        """删除用户权限

        参数:
            user_id: 用户id
            group_id: 群组id

        返回:
            bool: 是否含有用户权限
        """
        if user := await cls.get_or_none(user_id=user_id, group_id=group_id):
            await user.delete()
            return True
        return False

    @classmethod
    async def check_level(cls, user_id: str, group_id: str | None, level: int) -> bool:
        """检查用户权限等级是否大于 level

        参数:
            user_id: 用户id
            group_id: 群组id
            level: 权限等级

        返回:
            bool: 是否大于level
        """
        if group_id:
            if user := await cls.get_or_none(user_id=user_id, group_id=group_id):
                return user.user_level >= level
        else:
            if user_list := await cls.filter(user_id=user_id).all():
                user = max(user_list, key=lambda x: x.user_level)
                return user.user_level >= level
        return False

    @classmethod
    async def is_group_flag(cls, user_id: str, group_id: str) -> bool:
        """检测是否会被自动更新刷新权限

        参数:
            user_id: 用户id
            group_id: 群组id

        返回:
            bool: 是否会被自动更新权限刷新
        """
        if user := await cls.get_or_none(user_id=user_id, group_id=group_id):
            return user.group_flag == 1
        return False



from tortoise import fields

from ..db_connect import Model
from ..enum import LimitCheckType, LimitWatchType, PluginLimitType


class PluginLimit(Model):
    id = fields.IntField(pk=True, generated=True, auto_increment=True)
    """自增id"""
    module = fields.CharField(255, description="模块名")
    """模块名"""
    module_path = fields.CharField(255, description="模块路径")
    """模块路径"""
    plugin = fields.ForeignKeyField(
        "models.PluginInfo",
        related_name="plugin_limit",
        on_delete=fields.CASCADE,
        description="所属插件",
    )
    """所属插件"""
    limit_type = fields.CharEnumField(PluginLimitType, description="限制类型")
    """限制类型"""
    watch_type = fields.CharEnumField(LimitWatchType, description="监听类型")
    """监听类型"""
    status = fields.BooleanField(default=True, description="限制的开关状态")
    """限制的开关状态"""
    check_type = fields.CharEnumField(
        LimitCheckType, default=LimitCheckType.ALL, description="检查类型"
    )
    """检查类型"""
    result = fields.CharField(max_length=255, null=True, description="返回信息")
    """返回信息"""
    cd = fields.IntField(null=True, description="cd")
    """cd"""
    max_count = fields.IntField(null=True, description="最大调用次数")
    """最大调用次数"""

    class Meta:  # type: ignore
        table = "zxpm_plugin_limit"
        table_description = "插件限制"



from nonebot.rule import to_me
from nonebot_plugin_alconna import (
    Alconna,
    Args,
    Option,
    Subcommand,
    on_alconna,
    store_true,
)

from ...config import ZxpmConfig
from ...rules import admin_check, ensure_group

_status_matcher = on_alconna(
    Alconna(
        "switch",
        Option("-df|--default", action=store_true, help_text="进群默认开关"),
        Option("--all", action=store_true, help_text="全部插件/被动"),
        Option("-g|--group", Args["group?", str], help_text="指定群组"),
        Subcommand(
            "open",
            Args["plugin_name?", [str, int]],
        ),
        Subcommand(
            "close",
            Args["plugin_name?", [str, int]],
            Option(
                "-t|--type",
                Args["block_type?", ["all", "a", "private", "p", "group", "g"]],
            ),
        ),
    ),
    rule=admin_check(ZxpmConfig.zxpm_switch_level),
    priority=5,
    block=True,
)

_group_status_matcher = on_alconna(
    Alconna("group-status", Args["status", ["sleep", "wake"]]),
    rule=admin_check(ZxpmConfig.zxpm_switch_level) & ensure_group & to_me(),
    priority=5,
    block=True,
)

_status_matcher.shortcut(
    r"插件列表",
    command="switch",
    arguments=[],
    prefix=True,
)

_status_matcher.shortcut(
    r"开启(插件|功能)df(?P<name>.+)",
    command="switch",
    arguments=["open", "{name}", "-df"],
    prefix=True,
)

_status_matcher.shortcut(
    r"关闭(插件|功能)df(?P<name>.+)",
    command="switch",
    arguments=["close", "{name}", "-df"],
    prefix=True,
)


_status_matcher.shortcut(
    r"开启所有(插件|功能)",
    command="switch",
    arguments=["open", "s", "--all"],
    prefix=True,
)

_status_matcher.shortcut(
    r"开启所有(插件|功能)df",
    command="switch",
    arguments=["open", "s", "-df", "--all"],
    prefix=True,
)

_status_matcher.shortcut(
    r"开启(?P<name>.+)",
    command="switch",
    arguments=["open", "{name}"],
    prefix=True,
)


_status_matcher.shortcut(
    r"关闭所有(插件|功能)",
    command="switch",
    arguments=["close", "s", "--all"],
    prefix=True,
)

_status_matcher.shortcut(
    r"关闭所有(插件|功能)df",
    command="switch",
    arguments=["close", "s", "-df", "--all"],
    prefix=True,
)

_status_matcher.shortcut(
    r"关闭(?P<name>.+)",
    command="switch",
    arguments=["close", "{name}"],
    prefix=True,
)


_group_status_matcher.shortcut(
    r"醒来",
    command="group-status",
    arguments=["wake"],
    prefix=True,
)

_group_status_matcher.shortcut(
    r"休息吧",
    command="group-status",
    arguments=["sleep"],
    prefix=True,
)



from nonebot import require
from nonebot.plugin import PluginMetadata, inherit_supported_adapters

from .config import Config
from .enum import PluginType
from .extra import PluginExtraData

require("nonebot_plugin_alconna")
require("nonebot_plugin_session")
require("nonebot_plugin_uninfo")

from .commands import *  # noqa: F403

__plugin_meta__ = PluginMetadata(
    name="ZXPM插件管理",
    description="真寻的插件管理系统",
    usage="""
    包含了插件功能开关，群组/用户ban，群管监测，设置权限功能
    并提供一个简单的帮助接口，可以通过 zxpm [名称] 来获取帮助
    可以通过 -s 参数来获取该功能超级用户帮助
    例如：
        zxpm ban
        zxpm ban -s
    """,
    type="application",
    homepage="https://github.com/HibiKier/nonebot-plugin-zxpm",
    config=Config,
    supported_adapters=inherit_supported_adapters(
        "nonebot_plugin_alconna",
        "nonebot_plugin_session",
    ),
    extra=PluginExtraData(plugin_type=PluginType.PARENT).to_dict(),
)



from arclet.alconna import Args
from nonebot.adapters import Bot
from nonebot.permission import SUPERUSER
from nonebot.plugin import PluginMetadata
from nonebot_plugin_alconna import (
    Alconna,
    Arparma,
    At,
    Match,
    Option,
    on_alconna,
    store_true,
)
from nonebot_plugin_session import EventSession

from ...config import ZxpmConfig
from ...enum import PluginType
from ...extra import PluginExtraData
from ...log import logger
from ...rules import admin_check
from ...utils.utils import MessageUtils
from ._data_source import BanManage

__plugin_meta__ = PluginMetadata(
    name="Ban",
    description="你被逮捕了！丢进小黑屋！封禁用户以及群组，屏蔽消息",
    usage="""
    普通管理员
        格式:
        ban [At用户] ?[-t [时长(分钟)]]

        示例:
        ban @用户          : 永久拉黑用户
        ban @用户 -t 100   : 拉黑用户100分钟
        unban @用户        : 从小黑屋中拉出来
    """.strip(),
    extra=PluginExtraData(
        author="HibiKier",
        version="0.1",
        plugin_type=PluginType.SUPER_AND_ADMIN,
        admin_level=ZxpmConfig.zxpm_ban_level,
        superuser_help="""
        超级管理员额外命令
        格式:
        ban [At用户/用户Id] ?[-t [时长]]
        unban --id [idx]  : 通过id来进行unban操作
        ban列表: 获取所有Ban数据

        群组ban列表: 获取群组Ban数据
        用户ban列表: 获取用户Ban数据

        ban列表 -u [用户Id]: 查找指定用户ban数据
        ban列表 -g [群组Id]: 查找指定群组ban数据
        示例:
            ban列表 -u 123456789    : 查找用户123456789的ban数据
            ban列表 -g 123456789    : 查找群组123456789的ban数据

        私聊下:
            示例:
            ban 123456789          : 永久拉黑用户123456789
            ban 123456789 -t 100   : 拉黑用户123456789 100分钟

            ban -g 999999              : 拉黑群组为999999的群组
            ban -g 999999 -t 100       : 拉黑群组为999999的群组 100分钟

            unban 123456789     : 从小黑屋中拉出来
            unban -g 999999     : 将群组9999999从小黑屋中拉出来
        """,
    ).to_dict(),
)


_ban_matcher = on_alconna(
    Alconna(
        "ban",
        Args["user?", [str, At]],
        Option("-g|--group", Args["group_id", str]),
        Option("-t|--time", Args["duration", int]),
    ),
    rule=admin_check(ZxpmConfig.zxpm_ban_level),
    priority=5,
    block=True,
)

_unban_matcher = on_alconna(
    Alconna(
        "unban",
        Args["user?", [str, At]],
        Option("-g|--group", Args["group_id", str]),
        Option("--id", Args["idx", int]),
    ),
    rule=admin_check(ZxpmConfig.zxpm_ban_level),
    priority=5,
    block=True,
)

_status_matcher = on_alconna(
    Alconna(
        "ban列表",
        Option("-u", Args["user_id", str], help_text="查找用户"),
        Option("-g", Args["group_id", str], help_text="查找群组"),
        Option("--user", action=store_true, help_text="过滤用户"),
        Option("--group", action=store_true, help_text="过滤群组"),
    ),
    permission=SUPERUSER,
    priority=1,
    block=True,
)

_status_matcher.shortcut(
    "用户ban列表",
    command="ban列表",
    arguments=["--user"],
    prefix=True,
)

_status_matcher.shortcut(
    "群组ban列表",
    command="ban列表",
    arguments=["--group"],
    prefix=True,
)


@_status_matcher.handle()
async def _(
    arparma: Arparma,
    user_id: Match[str],
    group_id: Match[str],
):
    filter_type = None
    if arparma.find("user"):
        filter_type = "user"
    if arparma.find("group"):
        filter_type = "group"
    _user_id = user_id.result if user_id.available else None
    _group_id = group_id.result if group_id.available else None
    if image := await BanManage.build_ban_image(filter_type, _user_id, _group_id):
        await MessageUtils.build_message(image).finish(reply_to=True)
    else:
        await MessageUtils.build_message("数据为空捏...").finish(reply_to=True)


@_ban_matcher.handle()
async def _(
    bot: Bot,
    session: EventSession,
    arparma: Arparma,
    user: Match[str | At],
    duration: Match[int],
    group_id: Match[str],
):
    user_id = ""
    if not session.id1:
        await MessageUtils.build_message("用户id为空...").finish(reply_to=True)
    if user.available:
        if isinstance(user.result, At):
            user_id = user.result.target
        else:
            if session.id1 not in bot.config.superusers:
                await MessageUtils.build_message("权限不足捏...").finish(reply_to=True)
            user_id = user.result
    _duration = duration.result * 60 if duration.available else -1
    _duration_text = f"{duration.result} 分钟" if duration.available else " 到世界湮灭"
    if (gid := session.id3 or session.id2) and not group_id.available:
        if (
            not user_id
            or user_id == bot.self_id
            and session.id1 not in bot.config.superusers
        ):
            _duration = 0.5
            await MessageUtils.build_message("倒反天罡，小小管理速速退下！").send()
            await BanManage.ban(session.id1, gid, 30, session, True)
            _duration_text = "半 分钟"
            logger.info("尝试ban Bot 反被拿下", arparma.header_result, session=session)
            await MessageUtils.build_message(
                [
                    "对 ",
                    At(flag="user", target=session.id1),
                    " 狠狠惩戒了一番，一脚踢进了小黑屋!"
                    f" 在里面乖乖呆 {_duration_text}吧!",
                ]
            ).finish(reply_to=True)
        await BanManage.ban(
            user_id, gid, _duration, session, session.id1 in bot.config.superusers
        )
        logger.info(
            "管理员Ban",
            arparma.header_result,
            session=session,
            target=f"{gid}:{user_id}",
        )
        await MessageUtils.build_message(
            [
                "对 ",
                (
                    At(flag="user", target=user_id)
                    if isinstance(user.result, At)
                    else user_id
                ),  # type: ignore
                " 狠狠惩戒了一番，一脚踢进了小黑屋!",
                f" 在里面乖乖呆 {_duration_text} 吧!",
            ]
        ).finish(reply_to=True)
    elif session.id1 in bot.config.superusers:
        _group_id = group_id.result if group_id.available else None
        await BanManage.ban(user_id, _group_id, _duration, session, True)
        logger.info(
            "超级用户Ban",
            arparma.header_result,
            session=session,
            target=f"{_group_id}:{user_id}",
        )
        at_msg = user_id or f"群组:{_group_id}"
        await MessageUtils.build_message(
            f"对 {at_msg} 狠狠惩戒了一番，一脚踢进了小黑屋!"
        ).finish(reply_to=True)


@_unban_matcher.handle()
async def _(
    bot: Bot,
    session: EventSession,
    arparma: Arparma,
    user: Match[str | At],
    group_id: Match[str],
    idx: Match[int],
):
    user_id = ""
    _idx = idx.result if idx.available else None
    if user.available:
        if isinstance(user.result, At):
            user_id = user.result.target
        else:
            if session.id1 not in bot.config.superusers:
                await MessageUtils.build_message("权限不足捏...").finish(reply_to=True)
            user_id = user.result
    if gid := session.id3 or session.id2:
        if group_id.available:
            gid = group_id.result
        is_unban, result = await BanManage.unban(
            user_id, gid, session, _idx, session.id1 in bot.config.superusers
        )
        if not is_unban:
            await MessageUtils.build_message(result).finish(reply_to=True)
        logger.info(
            "管理员UnBan",
            arparma.header_result,
            session=session,
            target=f"{gid}:{result}",
        )
        await MessageUtils.build_message(
            [
                "将 ",
                (
                    At(flag="user", target=user_id)
                    if isinstance(user.result, At)
                    else result
                ),  # type: ignore
                " 从黑屋中拉了出来并急救了一下!",
            ]
        ).finish(reply_to=True)
    elif session.id1 in bot.config.superusers:
        _group_id = group_id.result if group_id.available else None
        is_unban, result = await BanManage.unban(
            user_id, _group_id, session, _idx, True
        )
        if not is_unban:
            await MessageUtils.build_message(result).finish(reply_to=True)
        logger.info(
            "超级用户UnBan",
            arparma.header_result,
            session=session,
            target=f"{_group_id}:{user_id}",
        )
        at_msg = user_id or f"群组:{result}"
        await MessageUtils.build_message(
            f"对 {at_msg} 从黑屋中拉了出来并急救了一下!"
        ).finish(reply_to=True)



import nonebot
from nonebot import get_loaded_plugins
from nonebot.drivers import Driver
from nonebot.plugin import Plugin, PluginMetadata
from ruamel.yaml import YAML

from ...enum import PluginType
from ...extra import PluginExtraData, PluginSetting
from ...log import logger
from ...models.plugin_info import PluginInfo
from ...models.plugin_limit import PluginLimit
from .manager import manager

_yaml = YAML(pure=True)
_yaml.allow_unicode = True
_yaml.indent = 2

driver: Driver = nonebot.get_driver()


async def _handle_setting(
    plugin: Plugin,
    plugin_list: list[PluginInfo],
    limit_list: list[PluginLimit],
):
    """处理插件设置

    参数:
        plugin: Plugin
        plugin_list: 插件列表
        limit_list: 插件限制列表
    """
    metadata = plugin.metadata
    if not metadata:
        if not plugin.sub_plugins:
            return
        """父插件"""
        metadata = PluginMetadata(name=plugin.name, description="", usage="")
    extra = metadata.extra
    extra_data = PluginExtraData(**extra)
    logger.debug(f"{metadata.name}:{plugin.name} -> {extra}", "初始化插件数据")
    setting = extra_data.setting or PluginSetting()
    if metadata.type == "library":
        extra_data.plugin_type = PluginType.HIDDEN
    if extra_data.plugin_type == PluginType.HIDDEN:
        extra_data.menu_type = ""
    if plugin.sub_plugins:
        extra_data.plugin_type = PluginType.PARENT
    plugin_list.append(
        PluginInfo(
            module=plugin.name,
            module_path=plugin.module_name,
            name=metadata.name,
            author=extra_data.author,
            version=extra_data.version,
            level=setting.level,
            default_status=setting.default_status,
            limit_superuser=setting.limit_superuser,
            menu_type=extra_data.menu_type,
            cost_gold=setting.cost_gold,
            plugin_type=extra_data.plugin_type,
            admin_level=extra_data.admin_level,
            parent=(plugin.parent_plugin.module_name if plugin.parent_plugin else None),
        )
    )
    if extra_data.limits:
        limit_list.extend(
            PluginLimit(
                module=plugin.name,
                module_path=plugin.module_name,
                limit_type=limit._type,
                watch_type=limit.watch_type,
                status=limit.status,
                check_type=limit.check_type,
                result=limit.result,
                cd=getattr(limit, "cd", None),
                max_count=getattr(limit, "max_count", None),
            )
            for limit in extra_data.limits
        )


@driver.on_startup
async def _():
    """
    初始化插件数据配置
    """
    plugin_list: list[PluginInfo] = []
    limit_list: list[PluginLimit] = []
    module2id = {}
    load_plugin = []
    if module_list := await PluginInfo.all().values("id", "module_path"):
        module2id = {m["module_path"]: m["id"] for m in module_list}
    for plugin in get_loaded_plugins():
        load_plugin.append(plugin.module_name)
        await _handle_setting(plugin, plugin_list, limit_list)
    create_list = []
    update_list = []
    for plugin in plugin_list:
        if plugin.module_path not in module2id:
            create_list.append(plugin)
        else:
            plugin.id = module2id[plugin.module_path]
            await plugin.save(
                update_fields=[
                    "name",
                    "author",
                    "version",
                    "admin_level",
                    "plugin_type",
                ]
            )
            update_list.append(plugin)
    if create_list:
        await PluginInfo.bulk_create(create_list, 10)
    await PluginInfo.filter(module_path__in=load_plugin).update(load_status=True)
    await PluginInfo.filter(module_path__not_in=load_plugin).update(load_status=False)
    manager.init()
    if limit_list:
        for limit in limit_list:
            if not manager.exist(limit.module_path, limit.limit_type):
                """不存在，添加"""
                manager.add(limit.module_path, limit)
    manager.save_file()
    await manager.load_to_db()



from nonebot.adapters.onebot.v11 import Bot, Event
from nonebot.matcher import Matcher
from nonebot.message import run_postprocessor, run_preprocessor
from nonebot_plugin_alconna import UniMsg
from nonebot_plugin_session import EventSession

from ._auth_checker import LimitManage, checker


# # 权限检测
@run_preprocessor
async def _(
    matcher: Matcher, event: Event, bot: Bot, session: EventSession, message: UniMsg
):
    await checker.auth(
        matcher,
        event,
        bot,
        session,
        message,
    )


# 解除命令block阻塞
@run_postprocessor
async def _(
    matcher: Matcher,
    exception: Exception | None,
    bot: Bot,
    event: Event,
    session: EventSession,
):
    user_id = session.id1
    group_id = session.id3
    channel_id = session.id2
    if not group_id:
        group_id = channel_id
        channel_id = None
    if user_id and matcher.plugin:
        module = matcher.plugin.name
        LimitManage.unblock(module, user_id, group_id, channel_id)



from nonebot.adapters import Bot, Event
from nonebot.internal.rule import Rule
from nonebot.permission import SUPERUSER
from nonebot_plugin_session import EventSession, SessionLevel

from .models.level_user import LevelUser


def ensure_group(session: EventSession) -> bool:
    """
    是否在群聊中

    参数:
        session: session

    返回:
        bool: bool
    """
    return session.level in [SessionLevel.LEVEL2, SessionLevel.LEVEL3]


def admin_check(a: int | None = None) -> Rule:
    """
    管理员权限等级检查

    参数:
        a: 权限等级或 配置项 module
        key: 配置项 key.

    返回:
        Rule: Rule
    """

    async def _rule(bot: Bot, event: Event, session: EventSession) -> bool:
        if await SUPERUSER(bot, event):
            return True
        if session.id1 and session.id2:
            level = a
            if level is not None:
                return bool(
                    await LevelUser.check_level(session.id1, session.id2, int(level))
                )
        return False

    return Rule(_rule)


def notice_rule(event_type: type | list[type]) -> Rule:
    """
    Notice限制

    参数:
        event_type: Event类型

    返回:
        Rule: Rule
    """

    async def _rule(event: Event) -> bool:
        if isinstance(event_type, list):
            for et in event_type:
                if isinstance(event, et):
                    return True
        else:
            return isinstance(event, event_type)
        return False

    return Rule(_rule)



from pathlib import Path

import nonebot
from pydantic import BaseModel

DEFAULT_DATA_PATH = Path() / "data" / "zxpm"


class Config(BaseModel):
    zxpm_data_path: str | Path = str(DEFAULT_DATA_PATH.absolute())
    """数据存储路径"""
    zxpm_db_url: str | None = None
    """DB_URL"""
    zxpm_notice_info_cd: int = 300
    """群/用户权限检测等各种检测提示信息cd，为0时不提醒"""
    zxpm_ban_reply: str = "才不会给你发消息."
    """用户被ban时回复消息，为空时不回复"""
    zxpm_ban_level: int = 5
    """使用ban功能的对应权限"""
    zxpm_switch_level: int = 1
    """群组插件开关管理对应权限"""
    zxpm_admin_default_auth: int = 5
    """群组管理员默认权限"""
    zxpm_font: str = "msyh.ttc"
    """字体"""


ZxpmConfig = nonebot.get_plugin_config(Config)

if isinstance(ZxpmConfig.zxpm_data_path, str):
    ZxpmConfig.zxpm_data_path = Path(ZxpmConfig.zxpm_data_path)
ZxpmConfig.zxpm_data_path.mkdir(parents=True, exist_ok=True)
if ZxpmConfig.zxpm_db_url:
    if ZxpmConfig.zxpm_db_url.startswith("sqlite"):
        db_path = ZxpmConfig.zxpm_db_url.split(":")[-1]
        Path(db_path).parent.mkdir(parents=True, exist_ok=True)
else:
    db_path = ZxpmConfig.zxpm_data_path / "db" / "zxpm.db"
    db_path.parent.mkdir(parents=True, exist_ok=True)
    ZxpmConfig.zxpm_db_url = f"sqlite:{db_path.absolute()!s}"



from nonebot.permission import SUPERUSER
from nonebot.plugin import PluginMetadata
from nonebot_plugin_alconna import (
    Alconna,
    Args,
    Arparma,
    At,
    Match,
    Option,
    Subcommand,
    on_alconna,
)
from nonebot_plugin_session import EventSession, SessionLevel

from ...enum import PluginType
from ...extra import PluginExtraData
from ...log import logger
from ...models.level_user import LevelUser
from ...utils.utils import MessageUtils

__plugin_meta__ = PluginMetadata(
    name="用户权限管理",
    description="设置用户权限",
    usage="""
    权限设置 add [level: 权限等级] [at: at对象或用户id] ?[-g gid: 群组]
    权限设置 delete [at: at对象或用户id]  ?[-g gid: 群组]

    添加权限 5 @user
    权限设置 add 5 422 -g 352352

    删除权限 @user
    删除权限 1234123 -g 123123

    """.strip(),
    extra=PluginExtraData(
        author="HibiKier",
        version="0.1",
        plugin_type=PluginType.SUPERUSER,
    ).dict(),
)


_matcher = on_alconna(
    Alconna(
        "权限设置",
        Subcommand(
            "add",
            Args["level", int]["uid", [str, At]],
            help_text="添加权限",
        ),
        Subcommand("delete", Args["uid", [str, At]], help_text="删除权限"),
        Option("-g|--group", Args["gid", str], help_text="指定群组"),
    ),
    permission=SUPERUSER,
    priority=5,
    block=True,
)

_matcher.shortcut(
    "添加权限",
    command="权限设置",
    arguments=["add", "{%0}"],
    prefix=True,
)

_matcher.shortcut(
    "删除权限",
    command="权限设置",
    arguments=["delete", "{%0}"],
    prefix=True,
)


@_matcher.assign("add")
async def _(
    session: EventSession,
    arparma: Arparma,
    level: int,
    gid: Match[str],
    uid: str | At,
):
    group_id = gid.result if gid.available else session.id3 or session.id2
    if group_id:
        if isinstance(uid, At):
            uid = uid.target
        user = await LevelUser.get_or_none(user_id=uid, group_id=group_id)
        old_level = user.user_level if user else 0
        await LevelUser.set_level(uid, group_id, level, 1)
        logger.info(
            f"修改权限: {old_level} -> {level}", arparma.header_result, session=session
        )
        if session.level in [SessionLevel.LEVEL2, SessionLevel.LEVEL3]:
            await MessageUtils.build_message(
                [
                    "成功为 ",
                    At(flag="user", target=uid),
                    f" 设置权限：{old_level} -> {level}",
                ]
            ).finish(reply_to=True)
        await MessageUtils.build_message(
            f"成功为 \n群组：{group_id}\n用户：{uid} \n"
            f"设置权限!\n权限：{old_level} -> {level}"
        ).finish()
    await MessageUtils.build_message("设置权限时群组不能为空...").finish()


@_matcher.assign("delete")
async def _(
    session: EventSession,
    arparma: Arparma,
    gid: Match[str],
    uid: str | At,
):
    group_id = gid.result if gid.available else session.id3 or session.id2
    if group_id:
        if isinstance(uid, At):
            uid = uid.target
        if user := await LevelUser.get_or_none(user_id=uid, group_id=group_id):
            await user.delete()
            if session.level in [SessionLevel.LEVEL2, SessionLevel.LEVEL3]:
                logger.info(
                    f"删除权限: {user.user_level} -> 0",
                    arparma.header_result,
                    session=session,
                )
                await MessageUtils.build_message(
                    ["成功删除 ", At(flag="user", target=uid), " 的权限等级!"]
                ).finish(reply_to=True)
            logger.info(
                f"删除群组用户权限: {user.user_level} -> 0",
                arparma.header_result,
                session=session,
            )
            await MessageUtils.build_message(
                f"成功删除 \n群组：{group_id}\n用户：{uid} \n"
                f"的权限等级!\n权限：{user.user_level} -> 0"
            ).finish()
        await MessageUtils.build_message("对方目前暂无权限喔...").finish()
    await MessageUtils.build_message("设置权限时群组不能为空...").finish()



from nonebot.compat import PYDANTIC_V2
from pydantic import BaseModel

from ..enum import PluginType
from .limit import BaseBlock, PluginCdBlock, PluginCountBlock


class PluginSetting(BaseModel):
    """
    插件基本配置
    """

    level: int = 5
    """群权限等级"""
    default_status: bool = True
    """进群默认开关状态"""
    limit_superuser: bool = False
    """是否限制超级用户"""
    cost_gold: int = 0
    """调用插件花费金币"""


class PluginExtraData(BaseModel):
    """
    插件扩展信息
    """

    author: str | None = None
    """作者"""
    version: str | None = None
    """版本"""
    plugin_type: PluginType = PluginType.NORMAL
    """插件类型"""
    menu_type: str = "功能"
    """菜单类型"""
    admin_level: int | None = None
    """管理员插件所需权限等级"""
    setting: PluginSetting | None = None
    """插件基本配置"""
    limits: list[BaseBlock | PluginCdBlock | PluginCountBlock] | None = None
    """插件限制"""
    superuser_help: str | None = None
    """超级用户帮助"""

    def to_dict(self):
        return self.model_dump() if PYDANTIC_V2 else self.dict()



from copy import deepcopy
from pathlib import Path

from ruamel.yaml import YAML

from ...config import ZxpmConfig
from ...enum import BlockType, LimitCheckType, PluginLimitType
from ...extra.limit import BaseBlock, PluginCdBlock, PluginCountBlock
from ...log import logger
from ...models.plugin_info import PluginInfo
from ...models.plugin_limit import PluginLimit

_yaml = YAML(pure=True)
_yaml.indent = 2
_yaml.allow_unicode = True


CD_TEST = """需要cd的功能
自定义的功能需要cd也可以在此配置
key：模块名称
cd：cd 时长（秒）
status：此限制的开关状态
check_type：'PRIVATE'/'GROUP'/'ALL'，限制私聊/群聊/全部
watch_type：监听对象，以user_id或group_id作为键来限制，'USER'：用户id，'GROUP'：群id
                                 示例：'USER':用户N秒内触发1次，'GROUP':群N秒内触发1次
result 为 "" 或 None 时则不回复
result示例："你冲的太快了，先生，请稍后再冲"
result回复："老色批你冲的太快了，欧尼酱先生，请稍后再冲@老色批"
"""


BLOCK_TEST = """用户调用阻塞
即 当用户调用此功能还未结束时
用发送消息阻止用户重复调用此命令直到该命令结束
key：模块名称
status：此限制的开关状态
check_type：'PRIVATE'/'GROUP'/'ALL'，限制私聊/群聊/全部
watch_type：监听对象，以user_id或group_id作为键来限制，'USER'：用户id，'GROUP'：群id
                                    示例：'USER'：阻塞用户，'group'：阻塞群聊
result 为 "" 或 None 时则不回复
result示例："你冲的太快了，先生，请稍后再冲"
result回复："老色批你冲的太快了，欧尼酱先生，请稍后再冲@老色批"
"""

COUNT_TEST = """命令每日次数限制
即 用户/群聊 每日可调用命令的次数 [数据内存存储，重启将会重置]
每日调用直到 00:00 刷新
key：模块名称
max_count: 每日调用上限
status：此限制的开关状态
watch_type：监听对象，以user_id或group_id作为键来限制，'USER'：用户id，'GROUP'：群id
                                     示例：'USER'：用户上限，'group'：群聊上限
result 为 "" 或 None 时则不回复
result示例："你冲的太快了，先生，请稍后再冲"
result回复："老色批你冲的太快了，欧尼酱先生，请稍后再冲@老色批"
"""


class Manager:
    """
    插件命令 cd 管理器
    """

    def __init__(self):
        if isinstance(ZxpmConfig.zxpm_data_path, str):
            ZxpmConfig.zxpm_data_path = Path(ZxpmConfig.zxpm_data_path)
        BASE_PATH = ZxpmConfig.zxpm_data_path / "configs"
        BASE_PATH.mkdir(parents=True, exist_ok=True)
        self.cd_file = BASE_PATH / "plugins2cd.yaml"
        self.block_file = BASE_PATH / "plugins2block.yaml"
        self.count_file = BASE_PATH / "plugins2count.yaml"
        self.cd_data = {}
        self.block_data = {}
        self.count_data = {}

    def add(
        self,
        module: str,
        data: BaseBlock | PluginCdBlock | PluginCountBlock | PluginLimit,
    ):
        """添加限制"""
        if isinstance(data, PluginLimit):
            check_type = BlockType.ALL
            if LimitCheckType.GROUP == data.check_type:
                check_type = BlockType.GROUP
            elif LimitCheckType.PRIVATE == data.check_type:
                check_type = BlockType.PRIVATE
            if data.limit_type == PluginLimitType.CD:
                data = PluginCdBlock(
                    status=data.status,
                    check_type=check_type,
                    watch_type=data.watch_type,
                    result=data.result,
                    cd=data.cd,
                )
            elif data.limit_type == PluginLimitType.BLOCK:
                data = BaseBlock(
                    status=data.status,
                    check_type=check_type,
                    watch_type=data.watch_type,
                    result=data.result,
                )
            elif data.limit_type == PluginLimitType.COUNT:
                data = PluginCountBlock(
                    status=data.status,
                    watch_type=data.watch_type,
                    result=data.result,
                    max_count=data.max_count,
                )
        if isinstance(data, PluginCdBlock):
            self.cd_data[module] = data
        elif isinstance(data, PluginCountBlock):
            self.count_data[module] = data
        elif isinstance(data, BaseBlock):
            self.block_data[module] = data

    def exist(self, module: str, type: PluginLimitType):
        """是否存在"""
        if type == PluginLimitType.CD:
            return module in self.cd_data
        elif type == PluginLimitType.BLOCK:
            return module in self.block_data
        elif type == PluginLimitType.COUNT:
            return module in self.count_data

    def init(self):
        if not self.cd_file.exists():
            self.save_cd_file()
        if not self.block_file.exists():
            self.save_block_file()
        if not self.count_file.exists():
            self.save_count_file()
        self.__load_file()

    def __load_file(self):
        self.__load_block_file()
        self.__load_cd_file()
        self.__load_count_file()

    def save_file(self):
        """保存文件"""
        self.save_cd_file()
        self.save_block_file()
        self.save_count_file()

    def save_cd_file(self):
        """保存文件"""
        self._extracted_from_save_file_3("PluginCdLimit", CD_TEST, self.cd_data)

    def save_block_file(self):
        """保存文件"""
        self._extracted_from_save_file_3(
            "PluginBlockLimit", BLOCK_TEST, self.block_data
        )

    def save_count_file(self):
        """保存文件"""
        self._extracted_from_save_file_3(
            "PluginCountLimit", COUNT_TEST, self.count_data
        )

    def _extracted_from_save_file_3(self, type_: str, after: str, data: dict):
        """保存文件

        参数:
            type_: 类型参数
            after: 备注
        """
        temp_data = deepcopy(data)
        if not temp_data:
            temp_data = {
                "test": {
                    "status": False,
                    "check_type": "ALL",
                    "limit_type": "USER",
                    "result": "你冲的太快了，请稍后再冲",
                }
            }
            if type_ == "PluginCdLimit":
                temp_data["test"]["cd"] = 5
            elif type_ == "PluginCountLimit":
                temp_data["test"]["max_count"] = 5
                del temp_data["test"]["check_type"]
        else:
            for v in temp_data:
                temp_data[v] = temp_data[v].dict()
                if check_type := temp_data[v].get("check_type"):
                    temp_data[v]["check_type"] = str(check_type)
                if watch_type := temp_data[v].get("watch_type"):
                    temp_data[v]["watch_type"] = str(watch_type)
                if type_ == "PluginCountLimit":
                    del temp_data[v]["check_type"]
        file = self.block_file
        if type_ == "PluginCdLimit":
            file = self.cd_file
        elif type_ == "PluginCountLimit":
            file = self.count_file
        with open(file, "w", encoding="utf8") as f:
            _yaml.dump({type_: temp_data}, f)
        with open(file, encoding="utf8") as rf:
            _data = _yaml.load(rf)
        _data.yaml_set_comment_before_after_key(after=after, key=type_)
        with open(file, "w", encoding="utf8") as wf:
            _yaml.dump(_data, wf)

    def __load_cd_file(self):
        self.cd_data: dict[str, PluginCdBlock] = {}
        if self.cd_file.exists():
            with open(self.cd_file, encoding="utf8") as f:
                temp = _yaml.load(f)
                if "PluginCdLimit" in temp.keys():
                    for k, v in temp["PluginCdLimit"].items():
                        self.cd_data[k] = PluginCdBlock.parse_obj(v)

    def __load_block_file(self):
        self.block_data: dict[str, BaseBlock] = {}
        if self.block_file.exists():
            with open(self.block_file, encoding="utf8") as f:
                temp = _yaml.load(f)
                if "PluginBlockLimit" in temp.keys():
                    for k, v in temp["PluginBlockLimit"].items():
                        self.block_data[k] = BaseBlock.parse_obj(v)

    def __load_count_file(self):
        self.count_data: dict[str, PluginCountBlock] = {}
        if self.count_file.exists():
            with open(self.count_file, encoding="utf8") as f:
                temp = _yaml.load(f)
                if "PluginCountLimit" in temp.keys():
                    for k, v in temp["PluginCountLimit"].items():
                        self.count_data[k] = PluginCountBlock.parse_obj(v)

    def __replace_data(
        self,
        db_data: PluginLimit | None,
        limit: PluginCdBlock | BaseBlock | PluginCountBlock,
    ) -> PluginLimit:
        """替换数据"""
        if not db_data:
            db_data = PluginLimit()
        db_data.status = limit.status
        check_type = LimitCheckType.ALL
        if BlockType.GROUP == limit.check_type:
            check_type = LimitCheckType.GROUP
        elif BlockType.PRIVATE == limit.check_type:
            check_type = LimitCheckType.PRIVATE
        db_data.check_type = check_type
        db_data.watch_type = limit.watch_type
        db_data.result = limit.result or ""
        return db_data

    def __set_data(
        self,
        k: str,
        db_data: PluginLimit | None,
        limit: PluginCdBlock | BaseBlock | PluginCountBlock,
        limit_type: PluginLimitType,
        module2plugin: dict[str, PluginInfo],
    ) -> tuple[PluginLimit, bool]:
        """设置数据

        参数:
            k: 模块名
            db_data: 数据库数据
            limit: 文件数据
            limit_type: 限制类型
            module2plugin: 模块:插件信息

        返回:
            tuple[PluginLimit, bool]: PluginLimit，是否创建
        """
        if not db_data:
            return (
                PluginLimit(
                    module=k,
                    module_path=module2plugin[k].module_path,
                    limit_type=limit_type,
                    plugin=module2plugin[k],
                    cd=getattr(limit, "cd", None),
                    max_count=getattr(limit, "max_count", None),
                    status=limit.status,
                    check_type=limit.check_type,
                    watch_type=limit.watch_type,
                    result=limit.result,
                ),
                True,
            )
        db_data = self.__replace_data(db_data, limit)
        if limit_type == PluginLimitType.CD:
            db_data.cd = limit.cd  # type: ignore
        if limit_type == PluginLimitType.COUNT:
            db_data.max_count = limit.max_count  # type: ignore
        return db_data, False

    def __get_file_data(self, limit_type: PluginLimitType) -> dict:
        """获取文件数据

        参数:
            limit_type: 限制类型

        返回:
            dict: 文件数据
        """
        if limit_type == PluginLimitType.CD:
            return self.cd_data
        elif limit_type == PluginLimitType.COUNT:
            return self.count_data
        else:
            return self.block_data

    def __set_db_limits(
        self,
        db_limits: list[PluginLimit],
        module2plugin: dict[str, PluginInfo],
        limit_type: PluginLimitType,
    ) -> tuple[list[PluginLimit], list[PluginLimit], list[int]]:
        """更新限制数据

        参数:
            db_limits: 数据库limits
            module2plugin: 模块:插件信息

        返回:
            tuple[list[PluginLimit], list[PluginLimit]]: 创建列表，更新列表
        """
        update_list = []
        create_list = []
        delete_list = []
        db_type_limits = [
            limit for limit in db_limits if limit.limit_type == limit_type
        ]
        if data := self.__get_file_data(limit_type):
            db_type_limit_modules = [
                (limit.module, limit.id) for limit in db_type_limits
            ]
            delete_list.extend(
                id for module, id in db_type_limit_modules if module not in data.keys()
            )
            for k, v in data.items():
                if not module2plugin.get(k):
                    if k != "test":
                        logger.warning(
                            f"插件模块 {k} 未加载，已过滤当前 {v._type} 限制..."
                        )
                    continue
                db_data = [limit for limit in db_type_limits if limit.module == k]
                db_data, is_create = self.__set_data(
                    k, db_data[0] if db_data else None, v, limit_type, module2plugin
                )
                if is_create:
                    create_list.append(db_data)
                else:
                    update_list.append(db_data)
        else:
            delete_list = [limit.id for limit in db_type_limits]
        return create_list, update_list, delete_list

    async def __set_all_limit(
        self,
    ) -> tuple[list[PluginLimit], list[PluginLimit], list[int]]:
        """获取所有插件限制数据

        返回:
            tuple[list[PluginLimit], list[PluginLimit]]: 创建列表，更新列表
        """
        db_limits = await PluginLimit.all()
        modules = set(
            list(self.cd_data.keys())
            + list(self.block_data.keys())
            + list(self.count_data.keys())
        )
        plugins = await PluginInfo.get_plugins(module__in=modules)
        module2plugin = {p.module: p for p in plugins}
        create_list, update_list, delete_list = self.__set_db_limits(
            db_limits, module2plugin, PluginLimitType.CD
        )
        create_list1, update_list1, delete_list1 = self.__set_db_limits(
            db_limits, module2plugin, PluginLimitType.COUNT
        )
        create_list2, update_list2, delete_list2 = self.__set_db_limits(
            db_limits, module2plugin, PluginLimitType.BLOCK
        )
        all_create = create_list + create_list1 + create_list2
        all_update = update_list + update_list1 + update_list2
        all_delete = delete_list + delete_list1 + delete_list2
        return all_create, all_update, all_delete

    async def load_to_db(self):
        """读取配置文件"""

        create_list, update_list, delete_list = await self.__set_all_limit()
        if create_list:
            await PluginLimit.bulk_create(create_list)
        if update_list:
            for limit in update_list:
                await limit.save(
                    update_fields=[
                        "status",
                        "check_type",
                        "watch_type",
                        "result",
                        "cd",
                        "max_count",
                    ]
                )
            # TODO: tortoise.exceptions.OperationalError:syntax error at or near "GROUP"
            # await PluginLimit.bulk_update(
            #     update_list,
            #     ["status", "check_type", "watch_type", "result", "cd", "max_count"],
            # )
        if delete_list:
            await PluginLimit.filter(id__in=delete_list).delete()
        cnt = await PluginLimit.filter(status=True).count()
        logger.info(f"已经加载 {cnt} 个插件限制.")


manager = Manager()



from nonebot.adapters import Bot, Event
from nonebot.exception import IgnoredException
from nonebot.matcher import Matcher
from nonebot.message import run_preprocessor
from nonebot.typing import T_State
from nonebot_plugin_alconna import At
from nonebot_plugin_session import EventSession

from ...config import ZxpmConfig
from ...extra import PluginType
from ...extra.limit import FreqLimiter
from ...log import logger
from ...models.ban_console import BanConsole
from ...models.group_console import GroupConsole
from ...utils.utils import MessageUtils

_flmt = FreqLimiter(300)


# 检查是否被ban
@run_preprocessor
async def _(
    matcher: Matcher, bot: Bot, event: Event, state: T_State, session: EventSession
):
    if plugin := matcher.plugin:
        if metadata := plugin.metadata:
            extra = metadata.extra
            if extra.get("plugin_type") in [PluginType.HIDDEN, PluginType.DEPENDANT]:
                return
    user_id = session.id1
    group_id = session.id3 or session.id2
    if group_id:
        if user_id in bot.config.superusers:
            return
        if await BanConsole.is_ban(None, group_id):
            logger.debug("群组处于黑名单中...", "ban_hook")
            raise IgnoredException("群组处于黑名单中...")
        if g := await GroupConsole.get_group(group_id):
            if g.level < 0:
                logger.debug("群黑名单, 群权限-1...", "ban_hook")
                raise IgnoredException("群黑名单, 群权限-1..")
    if user_id:
        ban_result = ZxpmConfig.zxpm_ban_reply
        if user_id in bot.config.superusers:
            return
        if await BanConsole.is_ban(user_id, group_id):
            time = await BanConsole.check_ban_time(user_id, group_id)
            if time == -1:
                time_str = "∞"
            else:
                time = abs(int(time))
                if time < 60:
                    time_str = f"{time!s} 秒"
                else:
                    minute = int(time / 60)
                    if minute > 60:
                        hours = minute // 60
                        minute %= 60
                        time_str = f"{hours} 小时 {minute}分钟"
                    else:
                        time_str = f"{minute} 分钟"
            if (
                time != -1
                and ban_result
                and _flmt.check(user_id)
                and ZxpmConfig.zxpm_ban_reply
            ):
                _flmt.start_cd(user_id)
                await MessageUtils.build_message(
                    [
                        At(flag="user", target=user_id),
                        f"{ban_result}\n在..在 {time_str} 后才会理你喔",
                    ]
                ).send()
            logger.debug("用户处于黑名单中...", "ban_hook")
            raise IgnoredException("用户处于黑名单中...")



from nonebot.adapters import Bot

from ...config import ZxpmConfig
from ...log import logger
from ...models.group_console import GroupConsole
from ...models.level_user import LevelUser
from ...models.plugin_info import PluginInfo


class GroupManager:
    @classmethod
    async def __handle_add_group(
        cls, bot: Bot, group_id: str, group: GroupConsole | None
    ):
        """允许群组并设置群认证，默认群功能开关

        参数:
            bot: Bot
            group_id: 群组id
            group: GroupConsole
        """
        block_plugin = ""
        if plugin_list := await PluginInfo.filter(default_status=False).all():
            for plugin in plugin_list:
                block_plugin += f"<{plugin.module},"
        group_info = await bot.get_group_info(group_id=group_id)
        if group:
            group.block_plugin = block_plugin
            await group.save(update_fields=["block_plugin"])
        else:
            await GroupConsole.create(
                group_id=group_info["group_id"],
                group_name=group_info["group_name"],
                max_member_count=group_info["max_member_count"],
                member_count=group_info["member_count"],
                block_plugin=block_plugin,
            )

    @classmethod
    async def __refresh_level(cls, bot: Bot, group_id: str):
        """刷新权限

        参数:
            bot: Bot
            group_id: 群组id
        """
        admin_default_auth = ZxpmConfig.zxpm_admin_default_auth
        member_list = await bot.get_group_member_list(group_id=group_id)
        member_id_list = [str(user_info["user_id"]) for user_info in member_list]
        flag2u = await LevelUser.filter(
            user_id__in=member_id_list, group_id=group_id
        ).values_list("user_id", flat=True)
        # 即刻刷新权限
        for user_info in member_list:
            user_id = user_info["user_id"]
            role = user_info["role"]
            if user_id in bot.config.superusers:
                await LevelUser.set_level(user_id, user_info["group_id"], 9)
                logger.debug(
                    "添加超级用户权限: 9",
                    "入群检测",
                    session=user_id,
                    group_id=user_info["group_id"],
                )
            elif (
                admin_default_auth is not None
                and role in ["owner", "admin"]
                and user_id not in flag2u
            ):
                await LevelUser.set_level(
                    user_id,
                    user_info["group_id"],
                    admin_default_auth,
                )
                logger.debug(
                    f"添加默认群管理员权限: {admin_default_auth}",
                    "入群检测",
                    session=user_id,
                    group_id=user_info["group_id"],
                )

    @classmethod
    async def add_bot(cls, bot: Bot, group_id: str, group: GroupConsole | None):
        """拉入bot

        参数:
            bot: Bot
            operator_id: 操作者id
            group_id: 群组id
            group: GroupConsole
        """
        await cls.__handle_add_group(bot, group_id, group)
        """刷新群管理员权限"""
        await cls.__refresh_level(bot, group_id)



from nonebot import on_notice
from nonebot.adapters.onebot.v11 import GroupAdminNoticeEvent
from nonebot.plugin import PluginMetadata

from ...config import ZxpmConfig
from ...enum import PluginType
from ...extra import PluginExtraData
from ...log import logger
from ...models.level_user import LevelUser
from ...rules import notice_rule

__plugin_meta__ = PluginMetadata(
    name="群管理员变动监测",
    description="""检测群管理员变动, 添加与删除管理员默认权限,
    当配置项 ADMIN_DEFAULT_AUTH 为空时, 不会添加管理员权限""",
    usage="",
    extra=PluginExtraData(
        author="HibiKier",
        version="0.1",
        plugin_type=PluginType.HIDDEN,
    ).to_dict(),
)


admin_notice = on_notice(priority=5, rule=notice_rule(GroupAdminNoticeEvent))


@admin_notice.handle()
async def _(event: GroupAdminNoticeEvent):
    if event.sub_type == "set":
        admin_default_auth = ZxpmConfig.zxpm_admin_default_auth
        if admin_default_auth is not None:
            await LevelUser.set_level(
                str(event.user_id),
                str(event.group_id),
                admin_default_auth,
            )
            logger.info(
                f"成为管理员，添加权限: {admin_default_auth}",
                "群管理员变动监测",
                session=event.user_id,
                group_id=event.group_id,
            )
        else:
            logger.warning(
                "配置项 MODULE: [<u><y>admin_bot_manage</y></u>] |"
                " KEY: [<u><y>ADMIN_DEFAULT_AUTH</y></u>] 为空"
            )
    elif event.sub_type == "unset":
        await LevelUser.delete_level(str(event.user_id), str(event.group_id))
        logger.info(
            "撤销群管理员, 取消权限等级",
            "群管理员变动监测",
            session=event.user_id,
            group_id=event.group_id,
        )



import contextlib

from nonebot.adapters import Bot, Event
from nonebot.exception import IgnoredException
from nonebot.matcher import Matcher
from nonebot_plugin_alconna import At, UniMsg
from nonebot_plugin_session import EventSession
from pydantic import BaseModel

from ...config import ZxpmConfig
from ...enum import BlockType, LimitWatchType, PluginLimitType, PluginType
from ...extra.limit import CountLimiter, FreqLimiter, UserBlockLimiter
from ...log import logger
from ...models.group_console import GroupConsole
from ...models.level_user import LevelUser
from ...models.plugin_info import PluginInfo
from ...models.plugin_limit import PluginLimit
from ...utils.utils import MessageUtils


class Limit(BaseModel):
    limit: PluginLimit
    limiter: FreqLimiter | UserBlockLimiter | CountLimiter

    class Config:
        arbitrary_types_allowed = True


class LimitManage:
    add_module = []  # noqa: RUF012

    cd_limit: dict[str, Limit] = {}  # noqa: RUF012
    block_limit: dict[str, Limit] = {}  # noqa: RUF012
    count_limit: dict[str, Limit] = {}  # noqa: RUF012

    @classmethod
    def add_limit(cls, limit: PluginLimit):
        """添加限制

        参数:
            limit: PluginLimit
        """
        if limit.module not in cls.add_module:
            cls.add_module.append(limit.module)
            if limit.limit_type == PluginLimitType.BLOCK:
                cls.block_limit[limit.module] = Limit(
                    limit=limit, limiter=UserBlockLimiter()
                )
            elif limit.limit_type == PluginLimitType.CD:
                cls.cd_limit[limit.module] = Limit(
                    limit=limit, limiter=FreqLimiter(limit.cd)
                )
            elif limit.limit_type == PluginLimitType.COUNT:
                cls.count_limit[limit.module] = Limit(
                    limit=limit, limiter=CountLimiter(limit.max_count)
                )

    @classmethod
    def unblock(
        cls, module: str, user_id: str, group_id: str | None, channel_id: str | None
    ):
        """解除插件block

        参数:
            module: 模块名
            user_id: 用户id
            group_id: 群组id
            channel_id: 频道id
        """
        if limit_model := cls.block_limit.get(module):
            limit = limit_model.limit
            limiter: UserBlockLimiter = limit_model.limiter  # type: ignore
            key_type = user_id
            if group_id and limit.watch_type == LimitWatchType.GROUP:
                key_type = channel_id or group_id
            limiter.set_false(key_type)

    @classmethod
    async def check(
        cls,
        module: str,
        user_id: str,
        group_id: str | None,
        channel_id: str | None,
        session: EventSession,
    ):
        """检测限制

        参数:
            module: 模块名
            user_id: 用户id
            group_id: 群组id
            channel_id: 频道id
            session: Session

        异常:
            IgnoredException: IgnoredException
        """
        if limit_model := cls.cd_limit.get(module):
            await cls.__check(limit_model, user_id, group_id, channel_id, session)
        if limit_model := cls.block_limit.get(module):
            await cls.__check(limit_model, user_id, group_id, channel_id, session)
        if limit_model := cls.count_limit.get(module):
            await cls.__check(limit_model, user_id, group_id, channel_id, session)

    @classmethod
    async def __check(
        cls,
        limit_model: Limit | None,
        user_id: str,
        group_id: str | None,
        channel_id: str | None,
        session: EventSession,
    ):
        """检测限制

        参数:
            limit_model: Limit
            user_id: 用户id
            group_id: 群组id
            channel_id: 频道id
            session: Session

        异常:
            IgnoredException: IgnoredException
        """
        if not limit_model:
            return
        limit = limit_model.limit
        limiter = limit_model.limiter
        is_limit = (
            LimitWatchType.ALL
            or (group_id and limit.watch_type == LimitWatchType.GROUP)
            or (not group_id and limit.watch_type == LimitWatchType.USER)
        )
        key_type = user_id
        if group_id and limit.watch_type == LimitWatchType.GROUP:
            key_type = channel_id or group_id
        if is_limit and not limiter.check(key_type):
            if limit.result:
                await MessageUtils.build_message(limit.result).send()
            logger.debug(
                f"{limit.module}({limit.limit_type}) 正在限制中...",
                "HOOK",
                session=session,
            )
            raise IgnoredException(f"{limit.module} 正在限制中...")
        else:
            if isinstance(limiter, FreqLimiter):
                limiter.start_cd(key_type)
            if isinstance(limiter, UserBlockLimiter):
                limiter.set_true(key_type)
            if isinstance(limiter, CountLimiter):
                limiter.increase(key_type)


class IsSuperuserException(Exception):
    pass


class AuthChecker:
    """
    权限检查
    """

    def __init__(self):
        check_notice_info_cd = ZxpmConfig.zxpm_notice_info_cd
        self._flmt = FreqLimiter(check_notice_info_cd)
        self._flmt_g = FreqLimiter(check_notice_info_cd)
        self._flmt_s = FreqLimiter(check_notice_info_cd)
        self._flmt_c = FreqLimiter(check_notice_info_cd)

    def is_send_limit_message(self, plugin: PluginInfo, sid: str) -> bool:
        """是否发送提示消息

        参数:
            plugin: PluginInfo

        返回:
            bool: 是否发送提示消息
        """
        if ZxpmConfig.zxpm_notice_info_cd == 0:
            return False
        if plugin.plugin_type == PluginType.DEPENDANT:
            return False
        return plugin.module != "ai" if self._flmt_s.check(sid) else False

    async def auth(
        self,
        matcher: Matcher,
        event: Event,
        bot: Bot,
        session: EventSession,
        message: UniMsg,
    ):
        """权限检查

        参数:
            matcher: matcher
            bot: bot
            session: EventSession
            message: UniMsg
        """
        is_ignore = False
        user_id = session.id1
        group_id = session.id3
        channel_id = session.id2
        if not group_id:
            group_id = channel_id
            channel_id = None
        with contextlib.suppress(ImportError):
            from nonebot.adapters.onebot.v11 import PokeNotifyEvent

            if matcher.type == "notice" and not isinstance(event, PokeNotifyEvent):
                """过滤除poke外的notice"""
                return
        if user_id and matcher.plugin and (module_path := matcher.plugin.module_name):
            if plugin := await PluginInfo.get_or_none(module_path=module_path):
                if plugin.plugin_type == PluginType.HIDDEN:
                    logger.debug("插件为HIDDEN，已跳过...")
                    return
                try:
                    if session.id1 not in bot.config.superusers:
                        await self.auth_group(plugin, session, message)
                        await self.auth_admin(plugin, session)
                        await self.auth_plugin(plugin, session, event)
                    await self.auth_limit(plugin, session)
                except IsSuperuserException:
                    logger.debug(
                        "超级用户或被ban跳过权限检测...", "HOOK", session=session
                    )
                except IgnoredException:
                    is_ignore = True
                    LimitManage.unblock(
                        matcher.plugin.name, user_id, group_id, channel_id
                    )
                except AssertionError as e:
                    is_ignore = True
                    logger.debug("消息无法发送", session=session, e=e)
        if is_ignore:
            raise IgnoredException("权限检测 ignore")

    async def auth_limit(self, plugin: PluginInfo, session: EventSession):
        """插件限制

        参数:
            plugin: PluginInfo
            session: EventSession
        """
        user_id = session.id1
        group_id = session.id3
        channel_id = session.id2
        if not group_id:
            group_id = channel_id
            channel_id = None
        limit_list: list[PluginLimit] = await plugin.plugin_limit.filter(
            status=True
        ).all()  # type: ignore
        for limit in limit_list:
            LimitManage.add_limit(limit)
        if user_id:
            await LimitManage.check(
                plugin.module, user_id, group_id, channel_id, session
            )

    async def auth_plugin(
        self, plugin: PluginInfo, session: EventSession, event: Event
    ):
        """插件状态

        参数:
            plugin: PluginInfo
            session: EventSession
        """
        group_id = session.id3
        channel_id = session.id2
        if not group_id:
            group_id = channel_id
            channel_id = None
        if user_id := session.id1:
            is_poke = False
            with contextlib.suppress(ImportError):
                from nonebot.adapters.onebot.v11 import PokeNotifyEvent

                is_poke = isinstance(event, PokeNotifyEvent)
            if group_id:
                sid = group_id or user_id
                if await GroupConsole.is_superuser_block_plugin(
                    group_id, plugin.module
                ):
                    """超级用户群组插件状态"""
                    if self.is_send_limit_message(plugin, sid) and not is_poke:
                        self._flmt_s.start_cd(group_id or user_id)
                        await MessageUtils.build_message(
                            "超级管理员禁用了该群此功能..."
                        ).send(reply_to=True)
                    logger.debug(
                        f"{plugin.name}({plugin.module}) 超级管理员禁用了该群此功能...",
                        "HOOK",
                        session=session,
                    )
                    raise IgnoredException("超级管理员禁用了该群此功能...")
                if await GroupConsole.is_normal_block_plugin(group_id, plugin.module):
                    """群组插件状态"""
                    if self.is_send_limit_message(plugin, sid) and not is_poke:
                        self._flmt_s.start_cd(group_id or user_id)
                        await MessageUtils.build_message("该群未开启此功能...").send(
                            reply_to=True
                        )
                    logger.debug(
                        f"{plugin.name}({plugin.module}) 未开启此功能...",
                        "HOOK",
                        session=session,
                    )
                    raise IgnoredException("该群未开启此功能...")
                if not plugin.status and plugin.block_type == BlockType.GROUP:
                    """全局群组禁用"""
                    try:
                        if self.is_send_limit_message(plugin, sid) and not is_poke:
                            self._flmt_c.start_cd(group_id)
                            await MessageUtils.build_message(
                                "该功能在群组中已被禁用..."
                            ).send(reply_to=True)
                    except Exception as e:
                        logger.error(
                            "auth_plugin 发送消息失败", "HOOK", session=session, e=e
                        )
                    logger.debug(
                        f"{plugin.name}({plugin.module}) 该插件在群组中已被禁用...",
                        "HOOK",
                        session=session,
                    )
                    raise IgnoredException("该插件在群组中已被禁用...")
            else:
                sid = user_id
                if not plugin.status and plugin.block_type == BlockType.PRIVATE:
                    """全局私聊禁用"""
                    try:
                        if self.is_send_limit_message(plugin, sid) and not is_poke:
                            self._flmt_c.start_cd(user_id)
                            await MessageUtils.build_message(
                                "该功能在私聊中已被禁用..."
                            ).send()
                    except Exception as e:
                        logger.error(
                            "auth_admin 发送消息失败", "HOOK", session=session, e=e
                        )
                    logger.debug(
                        f"{plugin.name}({plugin.module}) 该插件在私聊中已被禁用...",
                        "HOOK",
                        session=session,
                    )
                    raise IgnoredException("该插件在私聊中已被禁用...")
            if not plugin.status and plugin.block_type == BlockType.ALL:
                """全局状态"""
                if group_id and await GroupConsole.is_super_group(group_id):
                    raise IsSuperuserException()
                logger.debug(
                    f"{plugin.name}({plugin.module}) 全局未开启此功能...",
                    "HOOK",
                    session=session,
                )
                if self.is_send_limit_message(plugin, sid) and not is_poke:
                    self._flmt_s.start_cd(group_id or user_id)
                    await MessageUtils.build_message("全局未开启此功能...").send()
                raise IgnoredException("全局未开启此功能...")

    async def auth_admin(self, plugin: PluginInfo, session: EventSession):
        """管理员命令 个人权限

        参数:
            plugin: PluginInfo
            session: EventSession
        """
        user_id = session.id1
        if user_id and plugin.admin_level:
            if group_id := session.id3 or session.id2:
                if not await LevelUser.check_level(
                    user_id, group_id, plugin.admin_level
                ):
                    try:
                        if self._flmt.check(user_id):
                            self._flmt.start_cd(user_id)
                            await MessageUtils.build_message(
                                [
                                    At(flag="user", target=user_id),
                                    f"你的权限不足喔，"
                                    f"该功能需要的权限等级: {plugin.admin_level}",
                                ]
                            ).send(reply_to=True)
                    except Exception as e:
                        logger.error(
                            "auth_admin 发送消息失败", "HOOK", session=session, e=e
                        )
                    logger.debug(
                        f"{plugin.name}({plugin.module}) 管理员权限不足...",
                        "HOOK",
                        session=session,
                    )
                    raise IgnoredException("管理员权限不足...")
            elif not await LevelUser.check_level(user_id, None, plugin.admin_level):
                try:
                    await MessageUtils.build_message(
                        f"你的权限不足喔，该功能需要的权限等级: {plugin.admin_level}"
                    ).send()
                except Exception as e:
                    logger.error(
                        "auth_admin 发送消息失败", "HOOK", session=session, e=e
                    )
                logger.debug(
                    f"{plugin.name}({plugin.module}) 管理员权限不足...",
                    "HOOK",
                    session=session,
                )
                raise IgnoredException("权限不足")

    async def auth_group(
        self, plugin: PluginInfo, session: EventSession, message: UniMsg
    ):
        """群黑名单检测 群总开关检测

        参数:
            plugin: PluginInfo
            session: EventSession
            message: UniMsg
        """
        if not (group_id := session.id3 or session.id2):
            return
        text = message.extract_plain_text()
        group = await GroupConsole.get_group(group_id)
        if not group:
            """群不存在"""
            raise IgnoredException("群不存在")
        if group.level < 0:
            """群权限小于0"""
            logger.debug(
                "群黑名单, 群权限-1...",
                "HOOK",
                session=session,
            )
            raise IgnoredException("群黑名单")
        if not group.status:
            """群休眠"""
            if text.strip() != "醒来":
                logger.debug("群休眠状态...", "HOOK", session=session)
                raise IgnoredException("群休眠状态")
        if plugin.level > group.level:
            """插件等级大于群等级"""
            logger.debug(
                f"{plugin.name}({plugin.module}) 群等级限制.."
                f"该功能需要的群等级: {plugin.level}..",
                "HOOK",
                session=session,
            )
            raise IgnoredException(f"{plugin.name}({plugin.module}) 群等级限制...")


checker = AuthChecker()
